"""Urls for the emoticons demo"""
from django.conf.urls import url
from django.views.generic import TemplateView


urlpatterns = [
    url(r'^$', TemplateView.as_view(
        template_name='demo.html')),
]



"""Settings for the emoticons demo"""
import os

PROJECT_ROOT = os.path.abspath(os.path.dirname(__file__))

DEBUG = True

STATIC_URL = '/static/'

SECRET_KEY = 'secret-key'

ROOT_URLCONF = 'emoticons.demo.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [PROJECT_ROOT],
        'APP_DIRS': True,
    }
]

INSTALLED_APPS = [
    'emoticons',
    'django.contrib.staticfiles'
]



"""Settings for emoticons app"""
import os
import re
import binascii

from django.conf import settings
from django.template.loader import get_template


try:
    unicode = unicode
except NameError:  # Python 3
    unicode = str
    basestring = (str, bytes)


emoticons_default_list = (
    ((':)', ':-)', ':=)', '(smile)'), 'smile.gif'),
    ((':(', ':-(', ':=(', '(sad)'), 'sadsmile.gif'),
    ((';)', ';-)', ';=)', '(wink)'), 'wink.gif'),
    ((';(', ';-(', ';=(', '(cry)'), 'crying.gif'),
    ((':*', ':-*', ':=*', '(kiss)'), 'kiss.gif'),
    ((':|', ':-|', ':=|', '(speechless)'), 'speechless.gif'),
    ((':-?', ':?', ':=?', '(think)'), 'thinking.gif'),
    (('|(', '|-(', '|=(', '(dull)'), 'dull.gif'),
    (('|-)', 'I-)', 'I=)', '(snooze)'), 'sleepy.gif'),
    (('])', ']=)', '(grin)'), 'evilgrin.gif'),
    ((':$', ':-$', ':=$', '(blush)'), 'blush.gif'),
    ((':D', ':-D', ':=D', ':d', ':-d', ':=d', '(laugh)'), 'bigsmile.gif'),
    ((':O', ':-O', ':=O', ':o', ':-o', ':=o', '(surprised)'), 'surprised.gif'),
    (('8)', '8-)', '8=)', 'B)', 'B-)', 'B=)', '(cool)'), 'cool.gif'),
    (('8-|', 'B-|', '8|', 'B|', '8=|', 'B=|', '(nerd)'), 'nerd.gif'),
    ((':P', ':-P', ':=P', ':p', ':-p', ':=p', '(tongueout)'), 'tongueout.gif'),
    ((':S', ':s', ':-s', ':-S', ':=s', ':=S', '(worry)'), 'worried.gif'),
    ((':@', ':-@', ':=@', 'x(', 'x-(', 'X(', 'X-(', 'x=(', 'X=('),
     'angry.gif'),
    ((':x', ':-x', ':X', ':-X', ':#', ':-#', ':=x', ':=X', ':=#'),
     'lipssealed.gif'),
    ((':^)', '(wonder)'), 'wondering.gif'),
    (('\o/', '\:D/', '\:d/', '(dance)'), 'dance.gif'),
    (('(mm)', '(mmm)', '(mmmm)'), 'mmm.gif'),
    ('(hi)', 'hi.gif'),
    ('(call)', 'call.gif'),
    ('(devil)', 'devil.gif'),
    ('(angel)', 'angel.gif'),
    ('(envy)', 'envy.gif'),
    ('(wait)', 'wait.gif'),
    ('(yawn)', 'yawn.gif'),
    ('(puke)', 'puke.gif'),
    ('(sweat)', 'sweating.gif'),
    ('(doh)', 'doh.gif'),
    ('(talk)', 'talking.gif'),
    ('(inlove)', 'inlove.gif'),
    ('(party)', 'party.gif'),
    ('(wasntme)', 'itwasntme.gif'),
    ('(clap)', 'clapping.gif'),
    ('(rofl)', 'rofl.gif'),
    ('(whew)', 'whew.gif'),
    ('(happy)', 'happy.gif'),
    ('(smirk)', 'smirk.gif'),
    ('(nod)', 'nod.gif'),
    ('(shake)', 'shake.gif'),
    ('(punch)', 'punch.gif'),
    ('(emo)', 'emo.gif'),
    ('(sun)', 'sun.gif'),
    ('(music)', 'music.gif'),
    ('(coffee)', 'coffee.gif'),
    ('(beer)', 'beer.gif'),
    ('(ninja)', 'ninja.gif'),
    ('(bow)', 'bow.gif'),
    ('(mooning)', 'mooning.gif'),
    ('(finger)', 'middlefinger.gif'),
    ('(bandit)', 'bandit.gif'),
    ('(drunk)', 'drunk.gif'),
    ('(toivo)', 'toivo.gif'),
    ('(rock)', 'rock.gif'),
    ('(swear)', 'swear.gif'),
    ('(bug)', 'bug.gif'),
    ('(fubar)', 'fubar.gif'),
    ('(tmi)', 'tmi.gif'),
    ('(handshake)', 'handshake.gif'),
    (('(^)', '(cake)'), 'cake.gif'),
    (('(*)', '(star)'), 'star.gif'),
    (('(hug)', '(bear)'), 'bear.gif'),
    (('(makeup)', '(kate)'), 'makeup.gif'),
    (('(smoking)', '(smoke)'), 'smoke.gif'),
    (('(chuckle)', '(giggle)'), 'giggle.gif'),
    (('(headbang)', '(banghead)'), 'headbang.gif'),
    (('(pi)', '(pizza)'), 'pizza.gif'),
    (('(flex)', '(muscle)'), 'muscle.gif'),
    (('(cash)', '(mo)', '($)'), 'cash.gif'),
    (('(d)', '(D)', '(drink)'), 'drink.gif'),
    (('(e)', '(m)', '(mail)'), 'mail.gif'),
    (('(mp)', '(ph)', '(phone)'), 'phone.gif'),
    (('(poolparty)', '(hrv)'), 'poolparty.gif'),
    (('(F)', '(f)', '(flower)'), 'flower.gif'),
    (('(~)', '(film)', '(movie)'), 'movie.gif'),
    (('(rain)', '(st)', '(london)'), 'rain.gif'),
    (('(o)', '(O)', '(time)', '(clock)'), 'time.gif'),
    (('(y)', '(Y)', '(ok)', '(yes)'), 'yes.gif'),
    (('(n)', '(N)', '(ko)', '(no)'), 'no.gif'),
    (('(u)', '(U)', '(brokenheart)'), 'brokenheart.gif'),
    (('(h)', '(H)', '(l)', '(L)', '(heart)', '(love)'), 'heart.gif'),
)


def cast_to_list(emoticons_list):
    """
    Fix list of emoticons with a single name
    to a list for easier future iterations,
    and cast iterables to list.
    """
    emoticons_tuple = []
    for emoticons, image in emoticons_list:
        if isinstance(emoticons, basestring):
            emoticons = [emoticons]
        else:
            emoticons = list(emoticons)
        emoticons_tuple.append((emoticons, image))
    return emoticons_tuple


def compile_emoticons(emoticons_list):
    """
    Compile a new list of emoticon tuples.
    Each tuple contains a compiled regular expression
    of the emoticon, and the html version of the emoticon.
    """
    emoticons_compiled = []
    for emoticons, image in emoticons_list:
        for emoticon in emoticons:
            context = {
                'name': emoticon,
                'image': os.path.join(EMOTICONS_DIRECTORY, image),
                'code': binascii.hexlify(emoticon.encode('utf-8'))}
            emoticons_compiled.append(
                (re.compile(re.escape(emoticon)),
                 EMOTICON_TEMPLATE.render(context).strip())
            )
    return emoticons_compiled


EMOTICONS_LIST = getattr(settings, 'EMOTICONS_LIST',
                         emoticons_default_list)

EMOTICONS_LIST = cast_to_list(EMOTICONS_LIST)

EMOTICONS_DIRECTORY = getattr(settings, 'EMOTICONS_DIRECTORY',
                              'emoticons')

EMOTICON_TEMPLATE = get_template('emoticons/emoticon.html')

EMOTICONS_COMPILED = compile_emoticons(EMOTICONS_LIST)



"""Setup script for django-emoticons"""
import os

from setuptools import setup
from setuptools import find_packages

import emoticons

setup(
    name='django-emoticons',
    version=emoticons.__version__,

    description=('A usefull and incredible Django application '
                 'that allow you to use emoticons in your templates :)'),
    long_description=open(os.path.join('README.rst')).read(),
    keywords='django, emoticons, smiley',

    author=emoticons.__author__,
    author_email=emoticons.__email__,
    url=emoticons.__url__,

    license=emoticons.__license__,

    zip_safe=False,
    include_package_data=True,
    packages=find_packages(exclude=['emoticons.demo']),
    classifiers=[
        'Framework :: Django',
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'License :: OSI Approved :: BSD License',
        'Topic :: Software Development :: Libraries :: Python Modules'],
    install_requires=['beautifulsoup4']
)



"""Template tags for emoticons app"""
from bs4 import BeautifulSoup

from django import template
from django.utils.safestring import mark_safe
from django.utils.html import conditional_escape

from emoticons.settings import EMOTICONS_LIST
from emoticons.settings import EMOTICONS_COMPILED

register = template.Library()


def regexp_replace_emoticons(content):
    """
    Replace the emoticons string by HTML images,
    with regular expressions.
    """
    for emoticon, emoticon_html in EMOTICONS_COMPILED:
        if emoticon.search(content):
            content = emoticon.sub(emoticon_html, content)
    return content


def replace_emoticons(content, excluded_markups):
    """
    Replace the emoticons string by HTML images.
    If some markups should be excluded from replacement,
    BeautifulSoup will be used.
    """
    if not excluded_markups:
        return regexp_replace_emoticons(content)

    excluded_markups = excluded_markups.split(',') + ['[document]']
    soup = BeautifulSoup(content, 'html.parser')

    for content_string in list(soup.strings):
        if content_string.parent.name not in excluded_markups:
            replaced_content_string = regexp_replace_emoticons(content_string)
            if content_string != replaced_content_string:
                content_string.replace_with(
                    BeautifulSoup(replaced_content_string, 'html.parser'))
    return str(soup)


class EmoticonNode(template.Node):
    """
    Node for applying ``replace_emoticons`` on content.
    """
    def __init__(self, nodelist, exclude):
        self.nodelist = nodelist
        self.exclude = exclude
        if exclude:
            self.exclude_var = template.Variable(exclude)

    def render(self, context):
        content = self.nodelist.render(context)

        exclude = self.exclude
        if exclude:
            if exclude[0] == exclude[-1] and exclude[0] in ("'", '"'):
                exclude = exclude[1:-1]
            else:
                exclude = self.exclude_var.resolve(context)

        return replace_emoticons(content, exclude)


@register.tag('emoticons')
def emoticons_tag(parser, token):
    """
    Tag for rendering emoticons.
    """
    exclude = ''
    args = token.split_contents()
    if len(args) == 2:
        exclude = args[1]
    elif len(args) > 2:
        raise template.TemplateSyntaxError(
            'emoticons tag has only one optional argument')

    nodelist = parser.parse(['endemoticons'])
    parser.delete_first_token()
    return EmoticonNode(nodelist, exclude)


@register.filter('emoticons', needs_autoescape=True)
def emoticons_filter(content, exclude='', autoescape=None):
    """
    Filter for rendering emoticons.
    """
    esc = autoescape and conditional_escape or (lambda x: x)
    content = mark_safe(replace_emoticons(esc(content), exclude))
    return content


@register.simple_tag
def emoticons_index():
    """
    Display an tuple of list of available emoticons.
    """
    return list(zip(*EMOTICONS_LIST))[0]



"""Tests for emoticons app"""
from django.test import TestCase
from django.template import Context
from django.template import Template
from django.template import TemplateSyntaxError


class EmoticonsTestCase(TestCase):
    expected_result = (
        'Coding is fun <img class="emoticon emoticon-3a29" '
        'src="/emoticons/smile.gif" alt=":)" />.'
    )

    def assertEmoticons(self, html1, html2=None):
        if html2 is None:
            html2 = self.expected_result
        html1 = html1.strip()
        html2 = html2.strip()
        self.assertEquals(html1, html2)

    def test_filter(self):
        t = Template("""
        {% load emoticons_tags %}
        {{ content|emoticons }}
        """)
        html = t.render(Context({'content': 'Coding is fun :).'}))
        self.assertEmoticons(html)

    def test_filter_exclude(self):
        t = Template("""
        {% load emoticons_tags %}
        {{ content|safe|emoticons:'a,b' }}
        """)
        html = t.render(Context({
            'content': ('<p>Coding is fun :).</p>'
                        '<b>Coding is fun :).</b>'
                        '<a>Coding is fun :).</a>')}))
        self.assertEmoticons(
            html,
            '<p>Coding is fun <img alt=":)" '
            'class="emoticon emoticon-3a29" '
            'src="/emoticons/smile.gif"/>.</p>'
            '<b>Coding is fun :).</b>'
            '<a>Coding is fun :).</a>')

    def test_tag(self):
        t = Template("""
        {% load emoticons_tags %}
        {% emoticons %}
        Coding is fun :).
        {% endemoticons %}
        """)
        html = t.render(Context())
        self.assertEmoticons(html)

    def test_tag_invalid(self):
        with self.assertRaises(TemplateSyntaxError):
            Template("""
            {% load emoticons_tags %}
            {% emoticons to much args %}
            Coding is fun :).
            {% endemoticons %}
            """)

    def test_tag_exclude(self):
        t = Template("""
        {% load emoticons_tags %}
        {% emoticons 'b' %}
        <p>Coding is fun :).</p>
        <b>Coding is fun :).</b>
        <a>Coding is fun :).</a>
        {% endemoticons %}
        """)
        html = t.render(Context())
        self.assertEmoticons(
            html,
            '<p>Coding is fun <img alt=":)" '
            'class="emoticon emoticon-3a29" '
            'src="/emoticons/smile.gif"/>.</p>\n'
            '<b>Coding is fun :).</b>\n'
            '<a>Coding is fun <img alt=":)" '
            'class="emoticon emoticon-3a29" '
            'src="/emoticons/smile.gif"/>.</a>')
        t = Template("""
        {% load emoticons_tags %}
        {% emoticons 'a,b' %}
        <p>Coding is fun :).</p>
        <b>Coding is fun :).</b>
        <a>Coding is fun :).</a>
        {% endemoticons %}
        """)
        html = t.render(Context())
        self.assertEmoticons(
            html,
            '<p>Coding is fun <img alt=":)" '
            'class="emoticon emoticon-3a29" '
            'src="/emoticons/smile.gif"/>.</p>\n'
            '<b>Coding is fun :).</b>\n'
            '<a>Coding is fun :).</a>')

    def test_tag_exclude_var(self):
        t = Template("""
        {% load emoticons_tags %}
        {% emoticons exclude %}
        <p>Coding is fun :).</p>
        <b>Coding is fun :).</b>
        <a>Coding is fun :).</a>
        {% endemoticons %}
        """)
        html = t.render(Context({'exclude': 'p,a'}))
        self.assertEmoticons(
            html,
            '<p>Coding is fun :).</p>\n'
            '<b>Coding is fun <img alt=":)" '
            'class="emoticon emoticon-3a29" '
            'src="/emoticons/smile.gif"/>.</b>\n'
            '<a>Coding is fun :).</a>')

    def test_tag_exclude_with_inner_html(self):
        expected_html = ('<p>Fini l\'epoque du <em>Ctrl+F5</em> toutes '
                         'les 15 secondes. <img alt=":)" class="emoticon '
                         'emoticon-3a29" src="/emoticons/smile.gif"/></p>')
        t = Template("""
        {% load emoticons_tags %}
        {% emoticons "b" %}
        <p>Fini l'epoque du <em>Ctrl+F5</em> toutes les 15 secondes. :)</p>
        {% endemoticons %}
        """)
        self.assertEmoticons(t.render(Context()), expected_html)

        t = Template("""
        {% load emoticons_tags %}
        {{ content|safe|emoticons:"b" }}
        """)
        self.assertEmoticons(t.render(
            Context({'content': (
                "<p>Fini l'epoque du <em>Ctrl+F5</em> "
                "toutes les 15 secondes. :)</p>")})),
            expected_html)

    def test_tag_var(self):
        t = Template("""
        {% load emoticons_tags %}
        {% emoticons %}
        {{ content }}
        {% endemoticons %}
        """)
        html = t.render(Context({'content': 'Coding is fun :).'}))
        self.assertEmoticons(html)

    def test_multiple(self):
        t = Template("""
        {% load emoticons_tags %}
        {% emoticons %}
        {{ content }} :)
        {% endemoticons %}
        """)
        html = t.render(Context({'content': ':) :p'}))
        self.assertEmoticons(
            html,
            '<img class="emoticon emoticon-3a29" '
            'src="/emoticons/smile.gif" alt=":)" /> '
            '<img class="emoticon emoticon-3a70" '
            'src="/emoticons/tongueout.gif" alt=":p" /> '
            '<img class="emoticon emoticon-3a29" '
            'src="/emoticons/smile.gif" alt=":)" />')

    def test_index_assignment(self):
        t = Template("""
        {% load emoticons_tags %}
        {% emoticons_index as index  %}
        {{ index|length }}
        """)
        html = t.render(Context())
        self.assertEmoticons(html, '84')

    def test_index(self):
        t = Template("""
        {% load emoticons_tags %}
        {% emoticons_index %}
        """)
        html = t.render(Context())
        self.assertTrue(
            '([&#39;:)&#39;, &#39;:-)&#39;, &#39;:=)&#39;'
            ', &#39;(smile)&#39;]' in html)



"""Settings for testing emoticons"""

DATABASES = {
    'default': {'NAME': 'emoticons.db',
                'ENGINE': 'django.db.backends.sqlite3'}
}

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'APP_DIRS': True,
    }
]

INSTALLED_APPS = [
    'emoticons',
    'django.contrib.staticfiles'
]

SECRET_KEY = 'secret-key'

STATIC_URL = '/'
